
#include <Thor/Resources.hpp>
#include <SFML/Graphics.hpp>
#include <SFML/Audio.hpp>
#include <iostream>

int main()
{
	// Create texture and sound manager
	thor::ResourceManager<sf::Texture> textureMgr;
	thor::ResourceManager<sf::SoundBuffer> soundMgr;

	// Import names for simplicity
	using thor::Resources::TextureKey;
	using thor::Resources::SoundBufferKey;

	// Create sf::Image to load texture from
	sf::Image image;
	image.Create(529, 100, sf::Color(130, 70, 0));

	// Create keys to load resources
	TextureKey textureKey1  = TextureKey::FromImage(image);
	TextureKey textureKey2  = TextureKey::FromFile("Media/image.jpg");
	SoundBufferKey soundKey = SoundBufferKey::FromFile("Media/click.wav");

	// Create resource pointers to access the resources
	thor::ResourcePtr<sf::Texture>     texture1, texture2;
	thor::ResourcePtr<sf::SoundBuffer> soundBuffer;

	// Actually load resources, store them in resource pointers and react to loading errors
	try
	{
		texture1    = textureMgr.Acquire(textureKey1);
		texture2    = textureMgr.Acquire(textureKey2);
		soundBuffer = soundMgr.Acquire(soundKey);
	}
	catch (thor::ResourceLoadingException& e)
	{
		std::cout << e.what() << std::endl;
		return 1;
	}

	// Create instances that use the resources
	sf::Sprite sprite1(*texture1);
	sf::Sprite sprite2(*texture2);
	sf::Sound  sound(*soundBuffer);

	// Move second sprite so that the sprites don't overlap
	sprite2.Move( 0.f, static_cast<float>(texture1->GetHeight()) );

	// Create instruction text
	sf::Text instructions("Press return to play sound, escape to quit");
	instructions.SetCharacterSize(12);
	instructions.SetColor(sf::Color::White);

	// Create render window
	sf::RenderWindow window(sf::VideoMode(800, 600), "Thor Resources");
	window.SetVerticalSyncEnabled(true);

	for (;;)
	{
		// Handle events
		sf::Event event;
		while (window.PollEvent(event))
		{
			if (event.Type == sf::Event::Closed)
			{
				return 0;
			}
			else if (event.Type == sf::Event::KeyPressed)
			{
				switch (event.Key.Code)
				{
					case sf::Keyboard::Escape:
						return 0;

					case sf::Keyboard::Return:
						sound.Play();
						break;
				}
			}
		}

		// Draw sprites of the loaded textures
		window.Clear();
		window.Draw(sprite1);
		window.Draw(sprite2);
		window.Draw(instructions);
		window.Display();
	}
}